diff --git a/lisp/progmodes/compile.el b/lisp/progmodes/compile.el
index 9e441dbfcf7..79bf3f07f60 100644
--- a/lisp/progmodes/compile.el
+++ b/lisp/progmodes/compile.el
@@ -748,11 +748,39 @@ compilation-error-regexp-alist
 variable, the `compilation-transform-file-match-alist' variable
 is then consulted.  It allows further transformations of the
 matched file names, and weeding out false positives."
-  :type '(repeat (choice (symbol :tag "Predefined symbol")
-			 (sexp :tag "Error specification")))
+  ;; Segregate explicit rule specs and rule names in the list before we edit,
+  ;; to allow the names to be edited using a `set' widget.
+  :get (lambda (_)
+         (let ((specs nil) (syms nil))
+           (dolist (x compilation-error-regexp-alist)
+             (push x (if (symbolp x) syms specs)))
+           (append (nreverse specs) (nreverse syms))))
+  :type '(list :tag "Rules"
+               (repeat :tag "Custom message rules"
+                       :inline t
+                       (list :tag "Rule specification"
+                             regexp
+                             (sexp :tag "File")
+                             (sexp :tag "Line")
+                             (sexp :tag "Column")
+                             (sexp :tag "Type")
+                             (sexp :tag "Hyperlink")
+                             (repeat :inline t
+                                     :tag "Highlights"
+                                     (list :tag "Highlight"
+                                           (natnum :tag "Submatch")
+                                           (sexp :tag "Face")))))
+               (set :tag "Predefined rules"
+                    :inline t
+                    :convert-widget compilation--predef-rules-widgets))
   :link `(file-link :tag "example file"
 		    ,(expand-file-name "compilation.txt" data-directory)))
 
+(defun compilation--predef-rules-widgets (widget)
+  (widget-put widget :args (mapcar (lambda (x) (list 'const (car x)))
+                                   compilation-error-regexp-alist-alist))
+  widget)
+
 (defvar compilation-error-case-fold-search nil
   "If non-nil, use case-insensitive matching of compilation errors.
 If nil, matching is case-sensitive.
